from enum import Enum

from sqlmodel import SQLModel, Field, JSON

from xmpp_api.util.db import uuid


class JIDType(Enum):
    """
    The type of entity the JID is pointing to.
    """

    # JID points to an entity we can directly send messages to
    DIRECT = "DIRECT"

    # JID points to a MUC.
    GC = "GC"


class AllowedJid(SQLModel, table=True):
    # The relevant JID
    jid: str = Field(primary_key=True)

    # The type of entity the JID points to
    type: JIDType

    # The token that the client must use to specify the destination
    token: str

    # The bot this jid belongs to
    bot_id: str = Field(foreign_key="bot.id", primary_key=True)


class BotConstraint(SQLModel): ...


class BotDomainConstraint(BotConstraint):
    """
    Constraints the bot to send messages only to the provided domains
    """

    # List of domains that are allowed to send messages to
    domains: list[str]


class Bot(SQLModel, table=True):
    # The bot ID
    id: str = Field(default_factory=uuid, primary_key=True)

    # The bot's human readable name
    name: str = Field(unique=True)

    # The bot JID's localpart
    localpart: str = Field(unique=True)

    # The bot's description
    description: str | None

    # The bot's token
    token: str

    # Constraints on the bot
    constraints: list[BotConstraint] = Field(sa_type=JSON)

    # The owner the Bot belongs to
    owner_id: str = Field(foreign_key="user.id")
